﻿using System.Collections.Generic;
using System.Threading;
using System.Threading.Tasks;
using HeuristicLab.Algorithms.PushGP.Expressions;
using HeuristicLab.Algorithms.PushGP.Generators;
using HeuristicLab.Algorithms.PushGP.Stack;

namespace HeuristicLab.Algorithms.PushGP.Interpreter
{
    public interface IInterpreter
    {
        IStack<Expression> CodeStack { get; }
        IStack<Expression> ExecStack { get; }
        IStack<string> NameStack { get; }
        IStack<bool> BooleanStack { get; }
        IStack<long> IntegerStack { get; }
        IStack<double> FloatStack { get; }
        IDictionary<string, Expression> CustomExpressions { get; }

        Configuration Configuration { get; }

        CodeGenerator CodeGenerator { get; }

        bool IsNameQuoteFlagSet { get; set; }

        void Interpret(string code);
        Task InterpretAsync(string code, bool paused = false, CancellationToken token = default(CancellationToken));
        void Interpret(Expression program);
        Task InterpretAsync(Expression program, bool paused = false, CancellationToken token = default(CancellationToken));
        Task AbortAndResetAsync();
        Task AbortAsync();
        Task PauseAsync();
        Task ResumeAsync();
        void Step();
        void PrintStacks();
    }
}
